/*****************************************************************
*  jogoDaVelhaDicionario.c
* Bot utiliza dicionario de jogadas (similar a um sistema especialista
*    mas ao invs de regras todas as possibilidades j foram previamente
*    mapeadas -- 4520 jogadas/regras).
* Dicionario ocupa 38KB em disco (dicionario.txt) e a execucao
*    do jogo (que le o dicionario num formato interno, para facil acesso
*    mas com consumo maior de memoria, gasta, ao todo, 216KB de RAM).
******************************************************************
* Autor: Luciano Antonio Digiampietri
* Data : 02/07/2008 
*****************************************************************/

#include "jogoDaVelhaUtil.c"

#define MAX 19683

/* jogadas: vetor de elementos do tipo char que contem as melhores jogadas para
   cada estado do tabuleiro. As jogadas sao char de 0..8                      */
char jogadas[MAX];


/* inicializarJogadas: coloca o valor '9' em todas as jogadas (que eh um valor
      invalido                                                                */
void inicializarJogadas(){
	int i;
	for (i=0;i<MAX;i++) jogadas[i]='9';
} /* inicializarJogadas */

/* leDicionario: le o arquivo dicionario.txt, que contem a melhor 
     jogada para qualquer estado do tabuleiro de jogo da velha                */
void leDicionario(){
  FILE *f= fopen("dicionario.txt", "r");
  if (!f) {
    printf("Nao foi possivel abrir o arquivo de jogadas 'dicionario.txt'.\n");
    exit(1);
  }
  int indice;
  char valor;
  while(fscanf(f,"%d %c",&indice, &valor)!=EOF)  jogadas[indice] = valor;
  fclose(f);
} /* leDicionario */

/* indiceDeUmDadoTabuleiro: funcao que, dado um estado de um tabuleiro, retorna
     um indice correspondente a esse estado. Este indice sera usado para acessar
     o vetor de jogadas.                                                       */
int indiceDeUmDadoTabuleiro(JOGO *j){
	int i, valor[3];
	if (j->jogadorAtual == j1){
		for (i=0;i<3;i++) { 
			valor[i] = (j->A[3*i]+1)*9+(j->A[3*i+1]+1)*3+(j->A[3*i+2]+1)+1;
		}
	}else{
		for (i=0;i<3;i++) { 
			valor[i] = (j->A[3*i]*-1+1)*9+(j->A[3*i+1]*-1+1)*3+(j->A[3*i+2]*-1+1)+1;
		}
	}
	return valor[0] +27*(valor[1]+27*valor[2])-1;
} /* indiceDeUmDadoTabuleiro */


/* jogarDic: retorna a jogada oriunda do dicionario de jogadas, armazenado no 
   arranjo jogadas     */
JOGADA_VALOR jogarDic(JOGO * j){
	JOGADA_VALOR res;
	int i, valor[3];
	i = indiceDeUmDadoTabuleiro(j);
	if (jogadas[i]=='9') printf("Nao ha regra definida para o jogo atual: %d.\n",i);
	else{
		res.j = ((int) (jogadas[i]))-48;
		res.v = i;
	}
	return res;
} /* jogar */
