/*****************************************************************
 *      geracaoDeDicionarioUtil.c
 * Utiliza Busca Mini Max para gerar as melhores jogadas
 *    para cada estado do tabuleiro.
 ******************************************************************
 * Autor: Luciano Antonio Digiampietri
 * Data : 03/07/2008 
 *****************************************************************/

#include "jogoDaVelhaUtil.c"

#define MAX 19683

/* jogadas: vetor de elementos do tipo char que contem as melhores jogadas para
   cada estado do tabuleiro. As jogadas sao char de 0..8                      */
char jogadas[MAX];


/* inicializarJogadas: coloca o valor '9' em todas as jogadas (que eh um valor
      invalido                                                                */
void inicializarJogadas(){
	int i;
	for (i=0;i<MAX;i++) jogadas[i]='9';
} /* inicializarJogadas */


/* opcao da buscaMaxMin (se estah maximizando ou minimizando)                 */
typedef enum{max,min} opcao; 


/* Recebe como parametro a opcao max ou min; se opcao for max retorna o maximo
   entre a e b; caso contrario, retorna o valor minimo entre a e b.          */                                   
int maxMin(int a, int b, opcao op){
	if (((op==max) && a>b) || (op==min) && (a<b)) return a;
	return b;
}


/* retorna max caso a opcao de entra seja min e min caso contrario */
opcao outraOpcao(opcao op){
	if (op==max) return min;
	return max;
}

/* minMax: se opcao = min retorna o minimo entre 'a' e 'b', caso contrario retorna o
   maximo entre 'a' e 'b'  */
int minMax(int a, int b, opcao op){
	if ((op==min && a<b) || (op==max && a>b)) return a;
	return b;
} /* minMax */

/* indiceDeUmDadoTabuleiro: funcao que, dado um estado de um tabuleiro, retorna
     um indice correspondente a esse estado. Este indice sera usado para acessar
     o vetor de jogadas.                                                      */
int indiceDeUmDadoTabuleiro(JOGO *j){
	int i, valor[3];
	if (j->jogadorAtual == j1){
		for (i=0;i<3;i++) { 
			valor[i] = (j->A[3*i]+1)*9+(j->A[3*i+1]+1)*3+(j->A[3*i+2]+1)+1;
		}
	}else{
		for (i=0;i<3;i++) { 
			valor[i] = (j->A[3*i]*-1+1)*9+(j->A[3*i+1]*-1+1)*3+(j->A[3*i+2]*-1+1)+1;
		}
	}
	return valor[0] +27*(valor[1]+27*valor[2])-1;
} /* indiceDeUmDadoTabuleiro */

/* salvarEstadoRegra: dado o ponteiro para um jogo e a "melhor" jogada, salva
     no arranjo de jogadas a jogada atual para o indice correspondente ao estado
     atual do jogo.                                                           */
void salvarEstadoRegra(JOGO *j, JOGADA jog){
	int i;
	i = indiceDeUmDadoTabuleiro(j);
	jogadas[i] = (char)(jog+48); //ACII 48=0
} /* salvarEstadoRegra */

/* imprimirJogadas: imprime (na tela) o conjunto de todas as jogadas "salvas"
     ate o momento                                                            */
void imprimirJogadas(){
	int i;
	for (i=0;i<MAX;i++) {
		if (jogadas[i] != '9') printf("%d %c\n",i,jogadas[i]);
	}
} /* imprimirJogadas */

/* busca mini max (IA) para se selecionar a jogada atual do bot, minimizando
     a melhor jogada do adversario.                                           */
JOGADA_VALOR gerarJogadas(JOGO j, opcao op, int nivelAtual){
	nivelAtual++;
	LISTA_DE_JOGADAS jogadas = listaDeJogadas(&j); // posicao 0 guarda o numero de jogadas validas
	int valores[10];
	int jogadasValidas = jogadas.jogadas[0];
	int i;
	valores[0] = jogadasValidas;
	
	for (i=1;i<=jogadasValidas;i++){
		if (op == min){
		  valores[i] = 99999999;
		}else{
		  valores[i] = -99999999;
		}
	}
	for (i=1;i<=jogadasValidas;i++){
		JOGADA jogadaAtual = jogadas.jogadas[i];
		JOGO temp = j;
		jogador jogadorAtual = temp.jogadorAtual;
		inserirJogada(&temp,jogadaAtual,jogadorAtual);
		jogador vitorioso;
		bool fim = fimDeJogo(temp,&vitorioso);
		if (fim){
			if (vitorioso == empate){
				valores[i] = minMax(valores[i],0,op);
			}else {
				if (op==min) valores[i] = minMax(valores[i],-10+nivelAtual+1,min);
				else valores[i] = minMax(valores[i],10-nivelAtual-1,max);
			}
		}else{
			JOGADA_VALOR jv = gerarJogadas(temp,outraOpcao(op),nivelAtual);
			valores[i] = minMax(valores[i],jv.v,op);
		}
	}
	int jogadaSel = 1;
	int valorSel = valores[1];
	if (op==min){
		for (i=2;i<=jogadasValidas;i++){
			if (valores[i]<valorSel){
				jogadaSel=i;
				valorSel=valores[i];
			}
		}
	}else{
		for (i=2;i<=jogadasValidas;i++){
			if (valores[i]>valorSel){
				jogadaSel=i;
				valorSel=valores[i];
			}
		}
	}
	JOGADA_VALOR res;
	res.j = jogadas.jogadas[jogadaSel];
	res.v = valorSel;
    // apenas as jogadas com opcao min sao salvas 
	if (op==min) salvarEstadoRegra(&j,res.j);
	return res;
} /* gerarJogadas */
