/*****************************************************************
 *      jogoDaVelhaEspecialista.c
 * Bot utiliza um sistema especialista (baseado em regras), como 
 *    algoritmo de IA.
 ******************************************************************
 * Autor: Luciano Antonio Digiampietri
 * Data : 03/07/2008 
 *****************************************************************/

# include "jogoDaVelhaUtil.c"

/* retornaJogadaVitoriaDerrota: se jog==jogadorAtual retorna a jogada que faz
 *    o jogadorAtual ganhar (se houver) ou JOGADA_INVALIDA caso contrrio; se
 *    jog!=jogadorAtual, retorna a jogada que impede o jogadorAtual de perder
 *    ou JOGADA_INVALIDA caso contrrio.                                   */
JOGADA retornaJogadaVitoriaDerrota(JOGO j,jogador jog){
	/* uma jogada de vitoria ou derrota  caracterizada por duas marcas 
	 * seguidas com o mesmo valor e o terceiro valor livre                 */
	int valorAtual = valorDoJogador(j.jogadorAtual);
	if (jog!=j.jogadorAtual) valorAtual = -1*valorAtual;
	if ((j.A[0]+j.A[1]+j.A[2])==2*valorAtual){
		  if (j.A[0]==0) return 0;
		  else if (j.A[1]==0) return 1;
		  else return 2;
	  }
	  if ((j.A[3]+j.A[4]+j.A[5])==2*valorAtual){
		  if (j.A[3]==0) return 3;
		  else if (j.A[4]==0) return 4;
		  else return 5;
	  }
	  if ((j.A[6]+j.A[7]+j.A[8])==2*valorAtual){
		  if (j.A[6]==0) return 6;
		  else if (j.A[7]==0) return 7;
		  else return 8;
	  }
	  if ((j.A[0]+j.A[3]+j.A[6])==2*valorAtual){
		  if (j.A[0]==0) return 0;
		  else if (j.A[3]==0) return 3;
		  else return 6;
	  }
	  if ((j.A[1]+j.A[4]+j.A[7])==2*valorAtual){
		  if (j.A[1]==0) return 1;
		  else if (j.A[4]==0) return 4;
		  else return 7;
	  }
	  if ((j.A[2]+j.A[5]+j.A[8])==2*valorAtual){
		  if (j.A[2]==0) return 2;
		  else if (j.A[5]==0) return 5;
		  else return 8;
	  }
	  if ((j.A[2]+j.A[4]+j.A[6])==2*valorAtual){
		  if (j.A[2]==0) return 2;
		  else if (j.A[4]==0) return 4;
		  else return 6;
	  }
	  if ((j.A[0]+j.A[4]+j.A[8])==2*valorAtual){
		  if (j.A[0]==0) return 0;
		  else if (j.A[4]==0) return 4;
		  else return 8;
	  }
	  return JOGADA_INVALIDA;
} /* retornaJogadaVitoriaDerrota */

/* jogadaDoEspecialista: seleciona, de acordo com um conjunto de regras
 *    qual seria a jogada feita pelo especialista para um dado estado         */
JOGADA_VALOR jogadaDoEspecialista(JOGO j){
	JOGADA_VALOR res;
	
	// se nao houver jogadas retorna jogada invalida
	if (j.jogadas==9){
		res.j=-1;
		res.v=-1;
		return res;
	}
	
	// se houver uma jogada que te faa ganhar, jogue-a
	JOGADA jogadaVitoriosa = retornaJogadaVitoriaDerrota(j,j.jogadorAtual);
	if (jogadaVitoriosa != JOGADA_INVALIDA){
		res.j = jogadaVitoriosa;
		res.v = 100;
		return res;
	}

	// se houver uma jogada que te impea de perder, jogue-a
	JOGADA jogadaPrevenirDerrota = retornaJogadaVitoriaDerrota(j,outroJogador(j.jogadorAtual));
	if (jogadaPrevenirDerrota != JOGADA_INVALIDA){
		res.j = jogadaPrevenirDerrota;
		res.v = 5;
		return res;
	}

	// tenta prevenir a vitoria certa do adversario (que seria montar duas
	//   vitorias ao mesmo tempo).
	if (j.jogadas==3){
		if ((j.A[0]*j.A[8]==1 || j.A[2]*j.A[6]==1) && j.A[4]!=0){
			res.j=1;
			res.v=15;
			return res;			
		}
				
		if ((j.A[1]*j.A[8]==1 || j.A[0]*j.A[5]==1 || j.A[1]*j.A[5]==1) && j.A[4]!=0){
			res.j=2;
			res.v=20;
			return res;
		}
	}
	

	
	// tente jogar no meio ou nas diagonais (maior chance de vitoria ou impedir derrota
	if (j.jogadas <= 2){
		res.v=10;
		if (j.A[4]==0) res.j=4;
		else if (j.A[0]==0) res.j=0;
		else res.j=8;
		return res;
	}else{
		res.j = JOGADA_INVALIDA;
		if(j.A[4]==0) res.j=4;
		else if (j.A[6]==0) res.j=6;
		else if (j.A[8]==0) res.j=8;
		else if (j.A[0]==0) res.j=0;
		else if (j.A[2]==0) res.j=2;
		if (res.j!=JOGADA_INVALIDA){
			res.v=5;
			return res;
		}
	}
	
	LISTA_DE_JOGADAS jogadas = listaDeJogadas(&j); // posicao 0 guarda o numero de jogadas validas

	int jogadasValidas = jogadas.jogadas[0];

	// se s restar uma jogada vlida, faa-a
	if (jogadasValidas==1){
    	res.j = jogadas.jogadas[1];
    	res.v = 1;
    	return res;
	}
	
	// se nenhuma regra for atendida, faa a primeira jogada valida (poderia ser qualquer jogada valida)
	res.j = jogadas.jogadas[1];
	res.v = 0;		
	return res;
} /* jogadaDoEspecialista */


