/*****************************************************************
 *      jogoDaVelhaUtil.c
 * Possui toda a logica e estruturas para um jogo da velha.
 * Serve de base para o desenvolvimento de bots ou de um programa
 *   interativo para jogar jogo da velha.
 ******************************************************************
 * Autor: Luciano Antonio Digiampietri
 * Data : 03/07/2008 
 *****************************************************************/

#include <stdio.h>
#include <malloc.h>

#define v1 -1
#define v2 1
#define true 1
#define false 0
#define JOGADA_INVALIDA -1

typedef int JOGADA; // uma jogada  uma posio na matriz de 0 a 8

typedef struct {
	JOGADA jogadas[10];
}LISTA_DE_JOGADAS; // lista contendo ate 9 jogadas validas, jogadas[0] contem o numero de jogadas validas

typedef struct {
	JOGADA j;
	int v;
} JOGADA_VALOR; // par de uma jogada (posicao no tabuleiro) e o valor associado (buscaMaxMin)

typedef int bool;

typedef enum{j1,j2,empate} jogador; // tipos de jogadores e resultados

typedef struct {
	int A[9];
	jogador jogadorAtual;
	int jogadas;
} JOGO; // o "tabuleiro" do jogo da velha contendo o numero de jogadas realizadas e o jogador atual

/* inicializa / reinicializa um jogo da velha   */
void inicializarJogo(JOGO * j,jogador jog){
	int i;
	for(i=0;i<9;i++) j->A[i]=0;
	j->jogadorAtual=jog;
	j->jogadas=0;
}

/* retorna v1 caso jogador jog==j1 e v2 caso contrario   */
int valorDoJogador(jogador jog){
	if (jog==j1) return v1;
	return v2;
}

/* retorna j1 caso valor de entrada igual a v1 e j2 caso contrario */
jogador jogadorDoValor(int v){
	if (v==v1) return j1;
	return j2;
}

/* retorna j1 caso o jogador de entrada seja j2 e j1 caso contrario */
jogador outroJogador(jogador jog){
	if (jog==j1) return j2;
	return j1;
}

/* imprime o "tabuleiro" do jogo da velha */
void imprimirTabuleiro(JOGO j){
	int i;
	char desenho[3]; // contem o 'X' 'O' ou ' ' a ser desenhado no tabuleiro
    desenho[0] = 'X';
    desenho[1] = ' ';
    desenho[2] = 'O';
    for (i=0;i<3;i++){
		printf("  %c | %c | %c ",desenho[j.A[i*3]+1],desenho[j.A[i*3+1]+1],desenho[j.A[i*3+2]+1]);
        printf("     ||    %d | %d | %d",3*i,3*i+1,3*i+2);
        if (i == 2) printf("\n");
        else printf("\n -----------     ||   ----------- \n");
 	}
}

/* Funcao que verifica se o jogo acabou e retorna o jogador vitorioso no campo
   vitorioso. Retorna verdadeiro caso j1 ou j2 tenham ganho ou se o numero
   de jogadas seja igual a 9; e coloca j1, j2 ou empate no campo vitorioso.
   Caso contrario, retorna false.                                            */
bool fimDeJogo(JOGO j, jogador * vitorioso){
  if (abs(j.A[0]+j.A[1]+j.A[2])==3){
	  *vitorioso = jogadorDoValor(j.A[0]);
	  return true;
  }
  if (abs(j.A[3]+j.A[4]+j.A[5])==3){
	  *vitorioso = jogadorDoValor(j.A[3]);
	  return true;
  }
  if (abs(j.A[6]+j.A[7]+j.A[8])==3){
	  *vitorioso = jogadorDoValor(j.A[6]);
	  return true;
  }
  if (abs(j.A[0]+j.A[3]+j.A[6])==3){
	  *vitorioso = jogadorDoValor(j.A[0]);
	  return true;
  }
  if (abs(j.A[1]+j.A[4]+j.A[7])==3){
	  *vitorioso = jogadorDoValor(j.A[1]);
	  return true;
  }
  if (abs(j.A[2]+j.A[5]+j.A[8])==3){
	  *vitorioso = jogadorDoValor(j.A[2]);
	  return true;
  }
  if (abs(j.A[2]+j.A[4]+j.A[6])==3){
	  *vitorioso = jogadorDoValor(j.A[2]);
	  return true;
  }
  if (abs(j.A[0]+j.A[4]+j.A[8])==3){
	  *vitorioso = jogadorDoValor(j.A[0]);
	  return true;
  }
  if (j.jogadas == 9){
	  *vitorioso = empate;
	  return true;
  }
  return false;
}

/* Exibe o tabuleiro e, caso o jogo tenha acabado, o resultado do jogo */
void meuLog(JOGO j){
	imprimirTabuleiro(j);
	jogador vencedor = empate;
	if (fimDeJogo(j,&vencedor)){
		printf("Jogo encerrado. ");
		if (vencedor==empate) printf("Jogo empatado (velha).\n");
		else if (vencedor==j1) printf("Vitoria do jogador 1 (X).\n");
		  else printf("Vitoria do jogador 2 (O).\n");
	}
}

/* Recebe o ponteiro para um jogo, uma posicao e um jogador e verifica se a 
   jogada eh possivel. Se for faz a jogada do jogador jog na posicao 'posicao'
   e retorna true. Senao retorna false.                                      */
bool inserirJogada(JOGO * j, int posicao, jogador jog){
  jogador vitorioso;
  if (posicao<0 || posicao>9) return false;
  if (jog!=j->jogadorAtual) return false;
  if (fimDeJogo(*j,&vitorioso)) return false;
  if (j->jogadas==9) return false;
  if (j->A[posicao]!=0) return false;
  j->A[posicao]=valorDoJogador(jog);
  j->jogadas++;
  j->jogadorAtual = outroJogador(jog);
  return true;
}

/* retorna a lista de jogadas validas para o jogador atual (dado um jogo) */
LISTA_DE_JOGADAS listaDeJogadas(JOGO * j){
	LISTA_DE_JOGADAS resultado;
	resultado.jogadas[0]=0;
	jogador temp;
	if (!fimDeJogo(*j,&temp)){
	  int i;
	  for (i=0;i<9;i++){
		if (j->A[i] == 0){
			resultado.jogadas[0]++; // soma 1 nas jogadas vlidas
			resultado.jogadas[resultado.jogadas[0]] = i; // adiciona a posicao i como jogada valida
		}
	  }
	}
	return resultado;
}

